@@ -4,7 +4,6 @@ Generate models from httpx request responses.
 Also provides object mapping support to specific extracted item details
 """
 
-import asyncio
 import typing as t
 
 from moviebox_api._bases import (
@@ -25,6 +24,7 @@ from moviebox_api.extractor.models.json import ItemJsonDetailsModel
 from moviebox_api.helpers import (
     assert_instance,
     get_absolute_url,
+    get_event_loop,
     validate_item_page_url,
 )
 from moviebox_api.models import (
@@ -115,7 +115,7 @@ class BaseSearch(BaseContentProviderAndHelper):
                 return MovieDetails(item, self.session)
             case SubjectType.TV_SERIES:
                 return TVSeriesDetails(item, self.session)
-            case "_":
+            case _:
                 raise NotImplementedError(
                     f"Currently only items of {SubjectType.MOVIES.name} and {SubjectType.TV_SERIES.name} "
                     "subject-types are supported. Check later versions for possible support of other "
@@ -167,6 +167,19 @@ class Search(BaseSearch):
             dict: Fetched results
         """
         contents = await self.session.post_to_api(url=self._url, json=self._create_payload())
+
+        if self._subject_type is not SubjectType.ALL:
+            target_items = []
+
+            # Sometimes server response include irrelevant
+            # items
+
+            for item in contents["items"]:
+                if item["subjectType"] == self._subject_type.value:
+                    target_items.append(item)
+
+            contents["items"] = target_items
+
         return contents
 
     async def get_content_model(self) -> SearchResultsModel:
@@ -639,27 +652,23 @@ class BaseItemDetails(BaseContentProviderAndHelper):
         Returns:
             str: html formatted contents of the page
         """
-        return asyncio.get_event_loop().run_until_complete(self.get_html_content(*args, **kwargs))
+        return get_event_loop().run_until_complete(self.get_html_content(*args, **kwargs))
 
     def get_tag_details_extractor_sync(self, *args, **kwargs) -> TagDetailsExtractor:
         """Synchronously fetch content and return object that provide ways to extract details from html tags of the page"""  # noqa: E501
-        return asyncio.get_event_loop().run_until_complete(self.get_tag_details_extractor(*args, **kwargs))
+        return get_event_loop().run_until_complete(self.get_tag_details_extractor(*args, **kwargs))
 
     def get_json_details_extractor_sync(self, *args, **kwargs) -> JsonDetailsExtractor:
         """Synchronously fetch content and return object that extract details from json-formatted data in the page"""  # noqa: E501
-        return asyncio.get_event_loop().run_until_complete(self.get_json_details_extractor(*args, **kwargs))
+        return get_event_loop().run_until_complete(self.get_json_details_extractor(*args, **kwargs))
 
     def get_tag_details_extractor_model_sync(self, *args, **kwargs) -> TagDetailsExtractorModel:
         """Synchronously fetch content and return object that provide ways to model extracted details from html tags"""  # noqa: E501
-        return asyncio.get_event_loop().run_until_complete(
-            self.get_tag_details_extractor_model(*args, **kwargs)
-        )
+        return get_event_loop().run_until_complete(self.get_tag_details_extractor_model(*args, **kwargs))
 
     def get_json_details_extractor_model_sync(self, *args, **kwargs) -> JsonDetailsExtractorModel:
         """Synchronously fetch content and return object that models extracted details from json-formatted data in the page"""  # noqa: E501
-        return asyncio.get_event_loop().run_until_complete(
-            self.get_json_details_extractor_model(*args, **kwargs)
-        )
+        return get_event_loop().run_until_complete(self.get_json_details_extractor_model(*args, **kwargs))
 
 
 class MovieDetails(BaseItemDetails):
