@@ -2,6 +2,7 @@
 and later performing the actual download as well
 """
 
+import os
 from pathlib import Path
 
 import httpx
@@ -200,22 +201,25 @@ class MediaFileDownloader(BaseFileDownloaderAndHelper):
         part_dir: Path | str = CURRENT_WORKING_DIR,
         part_extension: str = DOWNLOAD_PART_EXTENSION,
         merge_buffer_size: int | None = None,
+        group_series: bool = False,
         **httpx_kwargs,
     ):
         """Constructor for `MediaFileDownloader`
 
         Args:
-            dir (Path | str, optional): Directory for downloaded files to. Defaults to CURRENT_WORKING_DIR.
+            dir (Path | str, optional): Directory for saving downloaded files to. Defaults to CURRENT_WORKING_DIR.
             chunk_size (int, optional): Streaming download chunk size in kilobytes. Defaults to DEFAULT_CHUNK_SIZE.
             tasks (int, optional): Number of tasks to carry out the download. Defaults to DEFAULT_TASKS.
             part_dir (Path | str, optional): Directory for temporarily saving downloaded file-parts to. Defaults to CURRENT_WORKING_DIR.
             part_extension (str, optional): Filename extension for download parts. Defaults to DOWNLOAD_PART_EXTENSION.
             merge_buffer_size (int|None, optional). Buffer size for merging the separated files in kilobytes. Defaults to chunk_size.
+            group_series(bool, optional): Create directory for a series & group episodes based on season number. Defaults to False.
 
         httpx_kwargs : Keyword arguments for `httpx.AsyncClient`
         """  # noqa: E501
 
         httpx_kwargs.setdefault("cookies", self.request_cookies)
+        self.group_series = group_series
 
         self.throttle_buster = ThrottleBuster(
             dir=dir,
@@ -228,15 +232,16 @@ class MediaFileDownloader(BaseFileDownloaderAndHelper):
             **httpx_kwargs,
         )
 
-    @classmethod
     def generate_filename(
-        cls,
+        self,
         search_results_item: SearchResultsItem,
         media_file: MediaFileMetadata,
         season: int = 0,
         episode: int = 0,
-    ) -> str:
-        """Generates filename in the format as in `self.*filename_template`
+        test: bool = False,
+    ) -> tuple[str, Path]:
+        """Generates filename in the format as in `self.*filename_template` and updates
+        final directory for saving contents
 
         Args:
             search_results_item (SearchResultsItem)
@@ -244,8 +249,6 @@ class MediaFileDownloader(BaseFileDownloaderAndHelper):
             season (int): Season number of the series.
             episde (int): Episode number of the series.
 
-        Returns:
-            str: Generated filename
         """
         assert_instance(
             search_results_item,
@@ -267,12 +270,21 @@ class MediaFileDownloader(BaseFileDownloaderAndHelper):
         )
 
         filename_template: str = (
-            cls.series_filename_template
+            self.series_filename_template
             if search_results_item.subjectType == SubjectType.TV_SERIES
-            else cls.movie_filename_template
+            else self.movie_filename_template
         )
 
-        return filename_template.format(**placeholders)
+        final_dir = self.create_final_dir(
+            working_dir=self.throttle_buster.dir,
+            search_results_item=search_results_item,
+            season=season,
+            episode=episode,
+            test=test,
+            group=self.group_series,
+        )
+
+        return filename_template.format(**placeholders), final_dir
 
     async def run(
         self,
@@ -316,9 +328,17 @@ class MediaFileDownloader(BaseFileDownloaderAndHelper):
 
         assert_instance(media_file, MediaFileMetadata, "media_file")
 
+        dir = None
+
         if isinstance(filename, SearchResultsItem):
-            filename = self.generate_filename(
-                search_results_item=filename, media_file=media_file, **filename_kwargs
+            filename, dir = self.generate_filename(
+                search_results_item=filename, media_file=media_file, test=test, **filename_kwargs
+            )
+
+        elif self.group_series:
+            raise ValueError(
+                f"Value for filename should be an instance of {SearchResultsItem} "
+                "when group_series is activated"
             )
 
         return await self.throttle_buster.run(
@@ -335,6 +355,7 @@ class MediaFileDownloader(BaseFileDownloaderAndHelper):
             test=test,
             leave=leave,
             ascii=ascii,
+            dir=dir,
         )
 
 
@@ -370,6 +391,7 @@ class CaptionFileDownloader(BaseFileDownloaderAndHelper):
         part_dir: Path | str = CURRENT_WORKING_DIR,
         part_extension: str = DOWNLOAD_PART_EXTENSION,
         merge_buffer_size: int | None = None,
+        group_series: bool = False,
         **httpx_kwargs,
     ):
         """Constructor for `CaptionFileDownloader`
@@ -380,11 +402,13 @@ class CaptionFileDownloader(BaseFileDownloaderAndHelper):
             part_dir (Path | str, optional): Directory for temporarily saving downloaded file-parts to. Defaults to CURRENT_WORKING_DIR.
             part_extension (str, optional): Filename extension for download parts. Defaults to DOWNLOAD_PART_EXTENSION.
             merge_buffer_size (int|None, optional). Buffer size for merging the separated files in kilobytes. Defaults to chunk_size.
+            group_series(bool, optional): Create directory for a series & group episodes based on season number. Defaults to False.
 
         httpx_kwargs : Keyword arguments for `httpx.AsyncClient`
         """  # noqa: E501
 
         httpx_kwargs.setdefault("cookies", self.request_cookies)
+        self.group_series = group_series
 
         self.throttle_buster = ThrottleBuster(
             dir=dir,
@@ -397,15 +421,15 @@ class CaptionFileDownloader(BaseFileDownloaderAndHelper):
             **httpx_kwargs,
         )
 
-    @classmethod
     def generate_filename(
-        cls,
+        self,
         search_results_item: SearchResultsItem,
         caption_file: CaptionFileMetadata,
         season: int = 0,
         episode: int = 0,
+        test: bool = False,
         **kwargs,
-    ) -> str:
+    ) -> tuple[str, Path]:
         """Generates filename in the format as in `self.*filename_template`
 
         Args:
@@ -413,13 +437,11 @@ class CaptionFileDownloader(BaseFileDownloaderAndHelper):
             caption_file (CaptionFileMetadata): Movie/tv-series/music caption file details.
             season (int): Season number of the series.
             episde (int): Episode number of the series.
+            test (bool, optional): whether to create final directory
 
         Kwargs: Nothing much folk.
                 It's just here so that `MediaFileDownloader.run` and `CaptionFileDownloader.run`
                 will accept similar parameters in `moviebox_api.extra.movies.Auto.run` method.
-
-        Returns:
-            str: Generated filename
         """
         assert_instance(
             search_results_item,
@@ -441,11 +463,21 @@ class CaptionFileDownloader(BaseFileDownloaderAndHelper):
         )
 
         filename_template: str = (
-            cls.series_filename_template
+            self.series_filename_template
             if search_results_item.subjectType == SubjectType.TV_SERIES
-            else cls.movie_filename_template
+            else self.movie_filename_template
         )
-        return sanitize_filename(filename_template.format(**placeholders))
+
+        final_dir = self.create_final_dir(
+            working_dir=self.throttle_buster.dir,
+            search_results_item=search_results_item,
+            season=season,
+            episode=episode,
+            test=test,
+            group=self.group_series,
+        )
+
+        return sanitize_filename(filename_template.format(**placeholders)), final_dir
 
     async def run(
         self,
@@ -471,12 +503,17 @@ class CaptionFileDownloader(BaseFileDownloaderAndHelper):
 
         assert_instance(caption_file, CaptionFileMetadata, "caption_file")
 
+        dir = None
+
         if isinstance(filename, SearchResultsItem):
             # Lets generate filename
-            filename = self.generate_filename(
+            filename, dir = self.generate_filename(
                 search_results_item=filename,
                 caption_file=caption_file,
                 season=season,
                 episode=episode,
+                test=run_kwargs.get("test", False),
             )
-        return await self.throttle_buster.run(url=str(caption_file.url), filename=filename, **run_kwargs)
+        return await self.throttle_buster.run(
+            url=str(caption_file.url), filename=filename, dir=dir, **run_kwargs
+        )
