@@ -3,6 +3,7 @@ This module provide functions for performing common and frequently required task
 across the package.
 """
 
+import asyncio
 import re
 import typing as t
 from urllib.parse import urljoin
@@ -15,7 +16,7 @@ FILE_EXT_PATTERN = re.compile(r".+\.(\w+)\?.+")
 
 ILLEGAL_CHARACTERS_PATTERN = re.compile(r"[^\w\-_\.\s()&|]")
 
-VALID_ITEM_PAGE_URL_PATTERN = re.compile(r".*" + ITEM_DETAILS_PATH + r"/[\w-]+\?id\=\d{19,}.*")
+VALID_ITEM_PAGE_URL_PATTERN = re.compile(r".*" + ITEM_DETAILS_PATH + r"/[\w-]+\?id\=\d{17,}.*")
 
 SCHEME_HOST_PATTERN = re.compile(r"https?://[-_\.\w]+")
 
@@ -30,7 +31,7 @@ def get_absolute_url(relative_url: str) -> str:
         str: Complete url with host
     """
 
-    return urljoin(HOST_URL, re.sub(SCHEME_HOST_PATTERN, "", relative_url))
+    return urljoin(HOST_URL, SCHEME_HOST_PATTERN.sub("", relative_url))
 
 
 def assert_membership(value: t.Any, elements: t.Iterable, identity="Value"):
@@ -81,17 +82,26 @@ def get_file_extension(url: str) -> str | None:
         url : https://valiw.hakunaymatata.com/resource/537977caa8c13703185d26471ce7de9f.mp4s?auth_key=1753024153-0-0-c824d3b5a5c8acc294bfd41de43c51ef"
         returns 'mp4'
     """
-    all = re.findall(FILE_EXT_PATTERN, str(url))
+    all = FILE_EXT_PATTERN.findall(str(url))
     if all:
         return all[0]
 
 
 def validate_item_page_url(url: str) -> str:
     """Checks whether specific item page url is valid"""
-    finds = re.findall(VALID_ITEM_PAGE_URL_PATTERN, url)
+    finds = VALID_ITEM_PAGE_URL_PATTERN.findall(url)
 
     if finds:
         if finds[0] == url:
             return url
 
     raise ValueError(f"Invalid url for a specific item page - '{url}'")
+
+
+def get_event_loop():
+    try:
+        event_loop = asyncio.get_event_loop()
+    except RuntimeError:
+        event_loop = asyncio.new_event_loop()
+        asyncio.set_event_loop(event_loop)
+    return event_loop
